/*
Section:  1) 0NF to 1NF
Script:   2) Getting Data Into SQL Server
Basics:   Given the Excel spreadsheet, we need some way of getting
			this data into SQL Server.  What follows is a coarse
			script which performs this action.
Notes:    If you do not have a database called "Scratch," this
			script will create one.
*/

--Create a Scratch database if we don't already have one.
IF NOT EXISTS
(
	SELECT 1
	FROM sys.databases
	WHERE
		name = 'Scratch'
)
BEGIN
	CREATE DATABASE Scratch;
END
GO

USE [Scratch]
GO

--Create all schemas if we don't already have them.
IF NOT EXISTS
(
	SELECT 1
	FROM sys.schemas
	WHERE
		name = '0NF'
)
BEGIN
	EXEC('CREATE SCHEMA [0NF]');
END
GO
IF NOT EXISTS
(
	SELECT 1
	FROM sys.schemas
	WHERE
		name = '1NF'
)
BEGIN
	EXEC('CREATE SCHEMA [1NF]');
END
GO
IF NOT EXISTS
(
	SELECT 1
	FROM sys.schemas
	WHERE
		name = '2NF'
)
BEGIN
	EXEC('CREATE SCHEMA [2NF]');
END
GO
IF NOT EXISTS
(
	SELECT 1
	FROM sys.schemas
	WHERE
		name = '3NF'
)
BEGIN
	EXEC('CREATE SCHEMA [3NF]');
END
GO
IF NOT EXISTS
(
	SELECT 1
	FROM sys.schemas
	WHERE
		name = 'BCNF'
)
BEGIN
	EXEC('CREATE SCHEMA [BCNF]');
END
GO
IF NOT EXISTS
(
	SELECT 1
	FROM sys.schemas
	WHERE
		name = '4NF'
)
BEGIN
	EXEC('CREATE SCHEMA [4NF]');
END
GO
IF NOT EXISTS
(
	SELECT 1
	FROM sys.schemas
	WHERE
		name = '5NF'
)
BEGIN
	EXEC('CREATE SCHEMA [5NF]');
END
GO

--Now look at the "1) King Crimson Lineups.xlsx" spreadsheet.  We want to get this into
--first normal form.  To do that, we need to satisfy four basic requirements:
--	1)  Records have a consistent shape
--	2)  All instances of an entity must be unique
--	3)  Attributes are atomic in nature
--	4)  No repeating groups of attributes exist
--Right off the bat, looking at the tabs, we see that there is NOT a consistent shape.
--This means that our spreadsheet is NOT in first normal form, and we cannot import
--it directly into a relational database system like this.
--Let's do the minimum necessary to get it into SQL Server, starting with table creation.

--Looking at each tab as a single table, we could end up with the following structure.
IF NOT EXISTS
(
	SELECT 1
	FROM sys.tables
	WHERE
		name = 'Lineup'
		AND OBJECT_SCHEMA_NAME(object_id) = '0NF'
)
BEGIN
	CREATE TABLE [0NF].[Lineup]
	(
		[Year] INT NOT NULL,
		[Member] VARCHAR(40) NOT NULL,
		[Instruments] VARCHAR(120) NOT NULL,
		[IsSessionOnly] BIT NOT NULL
	);
END
GO
IF NOT EXISTS
(
	SELECT 1
	FROM sys.tables
	WHERE
		name = 'Song'
		AND OBJECT_SCHEMA_NAME(object_id) = '0NF'
)
BEGIN
	CREATE TABLE [0NF].[Song]
	(
		[Year] INT NOT NULL,
		[Type] VARCHAR(6) NOT NULL,
		[Album] VARCHAR(120) NOT NULL,
		[Label] VARCHAR(50) NOT NULL,
		[Song] VARCHAR(120) NOT NULL,
		[Credits] VARCHAR(120) NOT NULL
	);
END
GO
IF NOT EXISTS
(
	SELECT 1
	FROM sys.tables
	WHERE
		name = 'ProjeKctTrack'
		AND OBJECT_SCHEMA_NAME(object_id) = '0NF'
)
BEGIN
	CREATE TABLE [0NF].[ProjeKctTrack]
	(
		[Year] INT NOT NULL,
		[Type] VARCHAR(6) NOT NULL,
		[Name] VARCHAR(40) NOT NULL,
		[Album] VARCHAR(120) NOT NULL,
		[Track] VARCHAR(120) NOT NULL,
		[Member1] VARCHAR(120) NOT NULL,
		[Member2] VARCHAR(120) NOT NULL,
		[Member3] VARCHAR(120) NULL,
		[Member4] VARCHAR(120) NULL,
		[Member5] VARCHAR(120) NULL,
		[Member6] VARCHAR(120) NULL
	);
END
GO

--Loading some sample data to show problems with 0NF.
MERGE [0NF].[Lineup] AS tgt
USING
(
	SELECT
		[Year],
		[Member],
		[Instruments],
		[IsSessionOnly]
	FROM
	(
		VALUES
			(1969, 'Robert Fripp', 'Guitar', 0),
			(1969, 'Ian McDonald', 'Reeds, Woodwind, Vibes Keyboards, Mellotron, Vocals', 0),
			(1969, 'Greg Lake', 'Bass, Lead Vocals', 0),
			(1969, 'Michael Giles', 'Drums, Percussion, Vocals', 0),
			(1969, 'Peter Sinfield', 'Words and Illumination', 0),
			(1970, 'Robert Fripp', 'Guitar, Mellotron, Devices', 0),
			(1970, 'Greg Lake', 'Vocals', 0),	
			(1970, 'Michael Giles', 'Drums', 0),
			(1970, 'Peter Giles', 'Bass', 0),
			(1970, 'Keith Trippett', 'Piano', 1),
			(1970, 'Mel Collins', 'Sax, Flute', 0),
			(1970, 'Gordon Haskell', 'Vocals', 0),
			(1970, 'Peter Sinfield', 'Words', 0)
	) v([Year], [Member], [Instruments], [IsSessionOnly])
) AS src
	ON src.[Year] = tgt.[Year]
	AND src.[Member] = tgt.[Member]
	AND src.[Instruments] = tgt.[Instruments]
	AND src.[IsSessionOnly] = tgt.[IsSessionOnly]
WHEN NOT MATCHED BY TARGET THEN INSERT
(
	[Year],
	[Member],
	[Instruments],
	[IsSessionOnly]
)
VALUES
(
	src.[Year],
	src.[Member],
	src.[Instruments],
	src.[IsSessionOnly]
);

MERGE [0NF].[ProjeKctTrack] AS tgt
USING
(
	SELECT
		[Year],
		[Type],
		[Name],
		[Album],
		[Track],
		[Member1],
		[Member2],
		[Member3],
		[Member4],
		[Member5],
		[Member6]
	FROM
	(
		VALUES
		(1998, 'Live', 'ProjeKct 1', 'Live at the Jazz Cafe', '4 i 1', 'Tony Levin - Bass, Stick, Synth', 'Trey Gunn - Touch guitar', 'Bill Bruford - Drums', 'Robert Fripp - Guitar', NULL, NULL),
		(1998, 'Live', 'ProjeKct 1', 'Live at the Jazz Cafe', '4 ii 1', 'Trey Gunn - Touch guitar', 'Tony Levin - Bass, Stick, Synth', 'Bill Bruford - Drums', 'Robert Fripp - Guitar', NULL, NULL),
		(1998, 'Live', 'ProjeKct 1', 'Live at the Jazz Cafe', '1 ii 2', 'Tony Levin - Bass, Stick, Synth', 'Trey Gunn - Touch guitar', 'Bill Bruford - Drums', 'Robert Fripp - Guitar', NULL, NULL),
		(1998, 'Live', 'ProjeKct 1', 'Live at the Jazz Cafe', '4 ii 4', 'Tony Levin - Bass, Stick, Synth', 'Trey Gunn - Touch guitar', 'Bill Bruford - Drums', 'Robert Fripp - Guitar', NULL, NULL),
		(1998, 'Live', 'ProjeKct 1', 'Live at the Jazz Cafe', '2 ii 3', 'Tony Levin - Bass, Stick, Synth', 'Trey Gunn - Touch guitar', 'Bill Bruford - Drums', 'Robert Fripp - Guitar', NULL, NULL),
		(1998, 'Live', 'ProjeKct 1', 'Live at the Jazz Cafe', '3 i 2', 'Tony Levin - Bass, Stick, Synth', 'Trey Gunn - Touch guitar', 'Bill Bruford - Drums', 'Robert Fripp - Guitar', NULL, NULL),
		(1998, 'Live', 'ProjeKct 1', 'Live at the Jazz Cafe', '3 ii 2', 'Tony Levin - Bass, Stick, Synth', 'Trey Gunn - Touch guitar', 'Bill Bruford - Drums', 'Robert Fripp - Guitar', NULL, NULL),
		(1998, 'Live', 'ProjeKct 1', 'Live at the Jazz Cafe', '2 ii 4', 'Tony Levin - Bass, Stick, Synth', 'Trey Gunn - Touch guitar', 'Bill Bruford - Drums', 'Robert Fripp - Guitar', NULL, NULL),
		(1998, 'Live', 'ProjeKct 1', 'Live at the Jazz Cafe', '4 i 3', 'Tony Levin - Bass, Stick, Synth', 'Trey Gunn - Touch guitar', 'Bill Bruford - Drums', 'Robert Fripp - Guitar', NULL, NULL),
		(1997, 'Live', 'ProjeKct 2', 'Live Groove', 'Sus-tayn-Z', 'Adrian Belew - V-Drums', 'Trey Gunn - Touch guitar, Talker', 'Robert Fripp - Guitar', NULL, NULL, NULL),
		(1997, 'Live', 'ProjeKct 2', 'Live Groove', 'Heavy ConstruKction', 'Adrian Belew - V-Drums', 'Trey Gunn - Touch guitar, Talker', 'Robert Fripp - Guitar', NULL, NULL, NULL),
		(1997, 'Live', 'ProjeKct 2', 'Live Groove', 'The Deception of the Thrush', 'Adrian Belew - V-Drums', 'Trey Gunn - Touch guitar, Talker', 'Robert Fripp - Guitar', NULL, NULL, NULL),
		(1997, 'Live', 'ProjeKct 2', 'Live Groove', 'X-chayn-jiZ', 'Adrian Belew - V-Drums', 'Trey Gunn - Touch guitar, Talker', 'Robert Fripp - Guitar', NULL, NULL, NULL),
		(1997, 'Live', 'ProjeKct 2', 'Live Groove', 'Light ConstruKction', 'Adrian Belew - V-Drums', 'Trey Gunn - Touch guitar, Talker', 'Robert Fripp - Guitar', NULL, NULL, NULL),
		(1997, 'Live', 'ProjeKct 2', 'Live Groove', 'Vector Shift to Planet Detroit', 'Adrian Belew - V-Drums', 'Trey Gunn - Touch guitar, Talker', 'Robert Fripp - Guitar', NULL, NULL, NULL),
		(1997, 'Live', 'ProjeKct 2', 'Live Groove', 'Contrary ConstruKction', 'Adrian Belew - V-Drums', 'Trey Gunn - Touch guitar, Talker', 'Robert Fripp - Guitar', NULL, NULL, NULL),
		(1997, 'Live', 'ProjeKct 2', 'Live Groove', 'Live Groove', 'Adrian Belew - V-Drums', 'Trey Gunn - Touch guitar, Talker', 'Robert Fripp - Guitar', NULL, NULL, NULL),
		(1997, 'Live', 'ProjeKct 2', 'Live Groove', 'Vector Shift to Planet Belewbeloid', 'Adrian Belew - V-Drums', 'Trey Gunn - Touch guitar, Talker', 'Robert Fripp - Guitar', NULL, NULL, NULL),
		(1997, 'Studio', 'ProjeKct 2', 'Space Groove', 'Space Groove II', 'Adrian Belew - V-Drums', 'Trey Gunn - Touch guitar, Guitar synth', 'Robert Fripp - Guitar', NULL, NULL, NULL),
		(1997, 'Studio', 'ProjeKct 2', 'Space Groove', 'Space Groove III', 'Adrian Belew - V-Drums', 'Trey Gunn - Touch guitar, Guitar synth', 'Robert Fripp - Guitar', NULL, NULL, NULL),
		(1997, 'Studio', 'ProjeKct 2', 'Space Groove', 'Space Groove I', 'Adrian Belew - V-Drums', 'Trey Gunn - Touch guitar, Guitar synth', 'Robert Fripp - Guitar', NULL, NULL, NULL),
		(1997, 'Studio', 'ProjeKct 2', 'Space Groove', 'The Planet Zarg Quartet', 'Adrian Belew - V-Drums', 'Trey Gunn - Touch guitar, Guitar synth', 'Robert Fripp - Guitar', NULL, NULL, NULL),
		(1997, 'Studio', 'ProjeKct 2', 'Space Groove', 'Happy Hour on Planet Zarg', 'Adrian Belew - V-Drums', 'Trey Gunn - Touch guitar, Guitar synth', 'Robert Fripp - Guitar', NULL, NULL, NULL),
		(1997, 'Studio', 'ProjeKct 2', 'Space Groove', 'Is There Life on Planet Zarg?', 'Adrian Belew - V-Drums', 'Trey Gunn - Touch guitar, Guitar synth', 'Robert Fripp - Guitar', NULL, NULL, NULL),
		(1997, 'Studio', 'ProjeKct 2', 'Space Groove', 'Low Life in Sector Q-3', 'Adrian Belew - V-Drums', 'Trey Gunn - Touch guitar, Guitar synth', 'Robert Fripp - Guitar', NULL, NULL, NULL),
		(1997, 'Studio', 'ProjeKct 2', 'Space Groove', 'Sector Shift', 'Adrian Belew - V-Drums', 'Trey Gunn - Touch guitar, Guitar synth', 'Robert Fripp - Guitar', NULL, NULL, NULL),
		(1997, 'Studio', 'ProjeKct 2', 'Space Groove', 'Laura in Space', 'Adrian Belew - V-Drums', 'Trey Gunn - Touch guitar, Guitar synth', 'Robert Fripp - Guitar', NULL, NULL, NULL),
		(1997, 'Studio', 'ProjeKct 2', 'Space Groove', 'Sector Drift', 'Adrian Belew - V-Drums', 'Trey Gunn - Touch guitar, Guitar synth', 'Robert Fripp - Guitar', NULL, NULL, NULL),
		(1997, 'Studio', 'ProjeKct 2', 'Space Groove', 'Sector Patrol', 'Adrian Belew - V-Drums', 'Trey Gunn - Touch guitar, Guitar synth', 'Robert Fripp - Guitar', NULL, NULL, NULL),
		(1997, 'Studio', 'ProjeKct 2', 'Space Groove', 'In Space There Is No North...', 'Adrian Belew - V-Drums', 'Trey Gunn - Touch guitar, Guitar synth', 'Robert Fripp - Guitar', NULL, NULL, NULL),
		(1997, 'Studio', 'ProjeKct 2', 'Space Groove', 'Vector Patrol', 'Adrian Belew - V-Drums', 'Trey Gunn - Touch guitar, Guitar synth', 'Robert Fripp - Guitar', NULL, NULL, NULL),
		(1997, 'Studio', 'ProjeKct 2', 'Space Groove', 'Deserts of Arcadia (North)', 'Adrian Belew - V-Drums', 'Trey Gunn - Touch guitar, Guitar synth', 'Robert Fripp - Guitar', NULL, NULL, NULL),
		(1997, 'Studio', 'ProjeKct 2', 'Space Groove', 'Deserts of Arcadia (South)', 'Adrian Belew - V-Drums', 'Trey Gunn - Touch guitar, Guitar synth', 'Robert Fripp - Guitar', NULL, NULL, NULL),
		(1997, 'Studio', 'ProjeKct 2', 'Space Groove', 'Snake Drummers of Sector Q-e', 'Adrian Belew - V-Drums', 'Trey Gunn - Touch guitar, Guitar synth', 'Robert Fripp - Guitar', NULL, NULL, NULL),
		(1997, 'Studio', 'ProjeKct 2', 'Space Groove', 'Escape from Sagittarius A', 'Adrian Belew - V-Drums', 'Trey Gunn - Touch guitar, Guitar synth', 'Robert Fripp - Guitar', NULL, NULL, NULL),
		(1997, 'Studio', 'ProjeKct 2', 'Space Groove', 'Return to Station B', 'Adrian Belew - V-Drums', 'Trey Gunn - Touch guitar, Guitar synth', 'Robert Fripp - Guitar', NULL, NULL, NULL)
	) v
	(
		[Year],
		[Type],
		[Name],
		[Album],
		[Track],
		[Member1],
		[Member2],
		[Member3],
		[Member4],
		[Member5],
		[Member6]
	)
) AS src
	ON tgt.[Year] = src.[Year]
	AND tgt.[Name] = src.[Name]
	AND tgt.[Album] = src.[Album]
	AND tgt.[Track] = src.[Track]
WHEN NOT MATCHED BY TARGET THEN INSERT
(
	[Year],
	[Type],
	[Name],
	[Album],
	[Track],
	[Member1],
	[Member2],
	[Member3],
	[Member4],
	[Member5],
	[Member6]
)
VALUES
(
	src.[Year],
	src.[Type],
	src.[Name],
	src.[Album],
	src.[Track],
	src.[Member1],
	src.[Member2],
	src.[Member3],
	src.[Member4],
	src.[Member5],
	src.[Member6]
);

--Issues that we can find in 0NF:
--1)  Provide a list of instruments Tony Levin played on ProjeKct 1. (No repeating groups)
--	This query seems overly complicated and we need to do LIKE operations 12 separate times,
--	not to mention 6 SUBSTRING operations and a DISTINCT operation.
SELECT DISTINCT
	CASE
		WHEN Member1 LIKE 'Tony Levin%' THEN SUBSTRING(Member1, 14, LEN(Member1))
		WHEN Member2 LIKE 'Tony Levin%' THEN SUBSTRING(Member2, 14, LEN(Member2))
		WHEN Member3 LIKE 'Tony Levin%' THEN SUBSTRING(Member3, 14, LEN(Member3))
		WHEN Member4 LIKE 'Tony Levin%' THEN SUBSTRING(Member4, 14, LEN(Member4))
		WHEN Member5 LIKE 'Tony Levin%' THEN SUBSTRING(Member5, 14, LEN(Member5))
		WHEN Member6 LIKE 'Tony Levin%' THEN SUBSTRING(Member6, 14, LEN(Member6))
		ELSE NULL
	END AS InstrumentList
FROM [0nf].[ProjeKctTrack] pt
WHERE
	pt.Name = 'ProjeKct 1'
	AND
	(
		pt.Member1 LIKE 'Tony Levin%'
		OR pt.Member2 LIKE 'Tony Levin%'
		OR pt.Member3 LIKE 'Tony Levin%'
		OR pt.Member4 LIKE 'Tony Levin%'
		OR pt.Member5 LIKE 'Tony Levin%'
		OR pt.Member6 LIKE 'Tony Levin%'
	);

--2)  Now return the above as a set; I need more than a comma-delimited list. (Atomic)
--		We use a tally table and a LOT of complex processing to return back a simple
--		list.  The fact that we're doing this much complex work for such a simple request
--		should tell us that there's a better way.
WITH E1(N) AS
(
    SELECT 1 UNION ALL SELECT 1 UNION ALL SELECT 1 UNION ALL
    SELECT 1 UNION ALL SELECT 1 UNION ALL SELECT 1 UNION ALL
    SELECT 1 UNION ALL SELECT 1 UNION ALL SELECT 1 UNION ALL SELECT 1
),                                      --10E+1 or 10 rows
E2(N) AS (SELECT 1 FROM E1 a, E1 b),    --10E+2 or 100 rows
cteTally(N) AS
(
     SELECT 0 UNION ALL
     SELECT TOP (100) ROW_NUMBER() OVER (ORDER BY (SELECT NULL))
        FROM E2
)
SELECT DISTINCT
	LTRIM(SUBSTRING(',' + il.InstrumentList + ',', c.N+2, CHARINDEX(',', ',' + il.InstrumentList + ',', N+2) - (N+2))) AS Instrument
FROM [0nf].[ProjeKctTrack] pt
	CROSS APPLY
	(
		SELECT DISTINCT
			CASE
				WHEN Member1 LIKE 'Tony Levin%' THEN SUBSTRING(Member1, 14, LEN(Member1))
				WHEN Member2 LIKE 'Tony Levin%' THEN SUBSTRING(Member2, 14, LEN(Member2))
				WHEN Member3 LIKE 'Tony Levin%' THEN SUBSTRING(Member3, 14, LEN(Member3))
				WHEN Member4 LIKE 'Tony Levin%' THEN SUBSTRING(Member4, 14, LEN(Member4))
				WHEN Member5 LIKE 'Tony Levin%' THEN SUBSTRING(Member5, 14, LEN(Member5))
				WHEN Member6 LIKE 'Tony Levin%' THEN SUBSTRING(Member6, 14, LEN(Member6))
				ELSE NULL
			END AS InstrumentList
	) il
	CROSS JOIN cteTally c
WHERE
	pt.Name = 'ProjeKct 1'
	AND
	(
		pt.Member1 LIKE 'Tony Levin%'
		OR pt.Member2 LIKE 'Tony Levin%'
		OR pt.Member3 LIKE 'Tony Levin%'
		OR pt.Member4 LIKE 'Tony Levin%'
		OR pt.Member5 LIKE 'Tony Levin%'
		OR pt.Member6 LIKE 'Tony Levin%'
	)
	AND
	(
		SUBSTRING(il.InstrumentList, c.N, 1) = ','
		OR c.N = 0
	);